@@ -13,10 +13,12 @@ from openmdao.api import Problem, Group, ExplicitComponent, ImplicitComponent, \
 from openmdao.core.tests.test_impl_comp import QuadraticLinearize, QuadraticJacVec
 from openmdao.core.tests.test_matmat import MultiJacVec
 from openmdao.test_suite.components.impl_comp_array import TestImplCompArrayMatVec
+from openmdao.test_suite.components.paraboloid import Paraboloid
 from openmdao.test_suite.components.paraboloid_mat_vec import ParaboloidMatVec
 from openmdao.test_suite.components.sellar import SellarDerivatives, SellarDis1withDerivatives, \
      SellarDis2withDerivatives
 from openmdao.test_suite.components.simple_comps import DoubleArrayComp
+from openmdao.test_suite.components.array_comp import ArrayComp
 from openmdao.test_suite.groups.parallel_groups import FanInSubbedIDVC
 from openmdao.utils.assert_utils import assert_rel_error, assert_warning, assert_check_partials
 from openmdao.utils.mpi import MPI
@@ -119,48 +121,6 @@ class MyComp(ExplicitComponent):
         J['y', 'x2'] = np.array([40])
 
 
-class ArrayComp(ExplicitComponent):
-
-    def setup(self):
-
-        J1 = np.array([[1.0, 3.0, -2.0, 7.0],
-                        [6.0, 2.5, 2.0, 4.0],
-                        [-1.0, 0.0, 8.0, 1.0],
-                        [1.0, 4.0, -5.0, 6.0]])
-
-        self.J1 = J1
-        self.J2 = J1 * 3.3
-        self.Jb = J1.T
-
-        # Inputs
-        self.add_input('x1', np.zeros([4]))
-        self.add_input('x2', np.zeros([4]))
-        self.add_input('bb', np.zeros([4]))
-
-        # Outputs
-        self.add_output('y1', np.zeros([4]))
-
-        self.declare_partials(of='*', wrt='*')
-        self.set_check_partial_options('x*', directional=True)
-
-        self.exec_count = 0
-
-    def compute(self, inputs, outputs):
-        """
-        Execution.
-        """
-        outputs['y1'] = self.J1.dot(inputs['x1']) + self.J2.dot(inputs['x2']) + self.Jb.dot(inputs['bb'])
-        self.exec_count += 1
-
-    def compute_partials(self, inputs, partials):
-        """
-        Analytical derivatives.
-        """
-        partials[('y1', 'x1')] = self.J1
-        partials[('y1', 'x2')] = self.J2
-        partials[('y1', 'bb')] = self.Jb
-
-
 class TestProblemCheckPartials(unittest.TestCase):
 
     def test_incorrect_jacobian(self):
@@ -1499,6 +1459,65 @@ class TestProblemCheckPartials(unittest.TestCase):
 
         assert_check_partials(data, atol=1.0E-8, rtol=1.0E-8)
 
+    def test_bug_local_method(self):
+        # This fixes a bug setting the check method on a component overrode the requested method for
+        # subsequent components.
+        prob = Problem()
+        model = prob.model
+
+        model.add_subsystem('comp1', Paraboloid())
+        fdcomp = model.add_subsystem('comp2', Paraboloid())
+        model.add_subsystem('comp3', Paraboloid())
+
+        fdcomp.set_check_partial_options(wrt='*', method='fd')
+
+        prob.setup(check=False, force_alloc_complex=True)
+        prob.set_solver_print(level=0)
+        prob.run_model()
+
+        data = prob.check_partials(method='cs', out_stream=None)
+
+        # Comp1 and Comp3 are complex step, so have tighter tolerances.
+        for key, val in iteritems(data['comp1']):
+            assert_rel_error(self, val['rel error'][0], 0.0, 1e-15)
+        for key, val in iteritems(data['comp2']):
+            assert_rel_error(self, val['rel error'][0], 0.0, 1e-6)
+        for key, val in iteritems(data['comp3']):
+            assert_rel_error(self, val['rel error'][0], 0.0, 1e-15)
+
+    def test_rel_error_fd_zero(self):
+        # When the fd turns out to be zero, test that we switch the definition of relative
+        # to divide by the forward derivative instead of reporting NaN.
+
+        class SimpleComp2(ExplicitComponent):
+            def setup(self):
+                self.add_input('x', val=3.0)
+                self.add_output('y', val=4.0)
+
+                self.declare_partials(of='y', wrt='x')
+
+            def compute(self, inputs, outputs):
+                # Mimics forgetting to set a variable.
+                pass
+
+            def compute_partials(self, inputs, partials):
+                partials['y', 'x'] = 3.0
+
+        prob = Problem()
+        prob.model = Group()
+
+        prob.model.add_subsystem('p1', IndepVarComp('x', 3.5))
+        prob.model.add_subsystem('comp', SimpleComp2())
+        prob.model.connect('p1.x', 'comp.x')
+
+        prob.setup(check=False)
+
+        stream = cStringIO()
+        data = prob.check_partials(out_stream=stream)
+        lines = stream.getvalue().splitlines()
+
+        self.assertTrue("Relative Error (Jfor  - Jfd) : 1." in lines[8])
+
 
 class TestCheckPartialsFeature(unittest.TestCase):
 
@@ -1739,6 +1758,7 @@ class TestCheckPartialsFeature(unittest.TestCase):
         prob.check_partials(step_calc='rel', compact_print=True)
 
     def test_feature_check_partials_show_only_incorrect(self):
+        import numpy as np
         from openmdao.api import Problem, Group, IndepVarComp, ExplicitComponent
 
         class MyCompGoodPartials(ExplicitComponent):
@@ -1820,6 +1840,8 @@ class TestCheckPartialsFeature(unittest.TestCase):
         prob.check_partials(compact_print=True, includes='*c*c*', excludes=['*e*'])
 
     def test_directional(self):
+        from openmdao.api import Problem
+        from openmdao.test_suite.components.array_comp import ArrayComp
 
         prob = Problem()
         model = prob.model
